
;(function (globalThis) {
    /*
     * 职责: 一个使用WebView模拟V8环境的脚本:
     * 1. 不修改KMA代码，将BindingAPI自动转成invokeAPI，将需要转的API名字加到callNativeHook列表里。
     * 2. 修复WebView无法使用loadScript问题, evaluateJavascript不能同步执行脚本。
     * 3. 兼容KSWorker环境
     */

    const isKSWorker = !!globalThis.importScripts;

    /**
     * KSJSCoreAPI转invoke列表
     * @type {*[]}
     */
    const callNativeHook = [
        'stats',
        'login',
        'getUserInfo',
        'authorize',
        'getLocation',
        'checkSession'
    ];

    /**
     * Bridge上方法转__make_async列表
     */
    const asyncHook = [
        'invokeHandler',
        'nativeLog'
    ];


    // 避免和kma的callbackId冲突
    let invokeCallbackId = 0x7fffffff >> 2;
    const callbackMap = {};
    const invokeName = 'bindMock';

    const empty = new Proxy({}, {
        get() {
            return {};
        }
    });

    function makeAsyncFunction(delegate, prop) {
        return function () {
            delegate.__make_async(prop, ...arguments);
        }
    }

    function proxy_invokeSyncHandler(name,params){
        var objparams = JSON.parse(params);
        if (objparams.action === 'getSystemInfo') {
            function formatjson(obj) {
                var returnValue = {};
                returnValue.result = 1;
                returnValue.data = {};
                returnValue.data.data = obj;
                returnValue.data.message = 'null:ok';
                return returnValue;
            }
            return formatjson(mini_app.getSystemInfoSync());
        }
        return __KWJSCoreBridge.invokeSyncHandler(name, params);
    }

    function proxyKWJSCoreBridge() {
        globalThis.KWJSCoreBridge = new Proxy(__KWJSCoreBridge, {
            get(target, p) {
                if (asyncHook.indexOf(p) >= 0) {
                    return makeAsyncFunction(__KWJSCoreBridge, p);
                }
               if(p === "invokeSyncHandler"){
                  return proxy_invokeSyncHandler;
               }
               return __KWJSCoreBridge[p].bind(__KWJSCoreBridge);
            }
        });
    }


    function proxyKWJSCoreAPI() {
        globalThis.KWJSCoreAPI = new Proxy(__KWJSCoreAPI, {
            get(target, p) {
                    const method = p;
                    if(method === "getAccountInfoSync"){
                        return (obj) => {
                         return mini_app.getAccountInfoSync();
                        }
                    }else if(method === "getLaunchOptionsSync"){
                         return (obj)  => {
                         return mini_app.getLaunchOptionsSync();
                        }
                    }else if (callNativeHook.indexOf(method) !== -1) {
                        return (obj) => {
                          console.log('[代理成InvokeAPI]', method, isSyncFunction(method));
                          return callInvoke(method, obj);
                        };
                    }else if (target[p]) {
                       return __KWJSCoreAPI[p].bind(__KWJSCoreAPI);
                    }
                    return () => {
                        console.error('[需要适配] KWJSCoreAPI 上没有这个方法 ', p);
                        return empty;
                    }
                }
            });
    }

    function isSyncFunction(method) {
        return method && method.indexOf('Sync') >= 0;
    }

    function defineLoadScriptSync() {
        // 定义loadScriptSync
        __KWJSCoreAPI.loadScriptSync = function (filePath) {
            console.log('调用loadScript', filePath, isKSWorker);
            if (isKSWorker) {
                globalThis.importScripts(filePath);
                return;
            }
            const script = document.createElement('script');
            script.innerHTML = KWJSCoreAPI.readFileSync(filePath, 'utf8');
            if (!script.innerHTML) {
                alert(filePath);
            }
            document.querySelector('head').appendChild(script);
        };
    }

    function formatData(data) {
        const {data: resObj, msg: errMsg} = data;
        const result = {
            errMsg,
        };
        if (typeof resObj === 'object' && !Array.isArray(resObj)) {
            Object.assign(result, resObj);
        } else {
            result.data = resObj;
        }
        return result;
    }

    function handleResponse({success, fail, complete}, {data, result}) {
        console.warn('[代理成InvokeAPI] 处理结果 data ', data, 'result is ', result);
        // format data
        data = formatData(data);

        if (result === 1) {
            success && success(data);
        } else {
            fail && fail(data);
        }
        complete && complete(data);
    }

    function invokeCallbackProxy(callbackId, response) {
        if (callbackMap[callbackId]) {
            console.warn('[代理成InvokeAPI] 处理invokeCallbackHandler ', callbackMap[callbackId].method);
            handleResponse(callbackMap[callbackId], response);
            delete callbackMap[callbackId];
            return;
        }
        return globalThis.__KSJSBridge['invokeCallbackHandler'](callbackId, response);
    }


    function callInvoke(method, object) {
        lazyProxyKSJSBridge();
        const {success, fail, complete, ...newParams} = object || {};
        const params = {
            name: invokeName,
            action: method,
            params: newParams
        };
        const paramsString = JSON.stringify(params);
        if (isSyncFunction(method)) {
            let resultString = KWJSCoreBridge.invokeSyncHandler('native.api', paramsString);
            let resultObject = JSON.parse(resultString);
            return resultObject && resultObject.data && resultObject.data.data;
        }
        console.warn('callInvoke msg is', params);
        const currentCallbackId = ++invokeCallbackId;
        callbackMap[currentCallbackId] = {success, fail, complete, method};
        try {
            KWJSCoreBridge.invokeHandler('native.api', paramsString, currentCallbackId);
        } catch (e) {
            console.error('[出现异常] invoke 调用时出现异常，参数是', paramsString);
        }
    }

    /**
     * KSJSBridge定义的较晚，用于接收invoke的回调
     */
    function lazyProxyKSJSBridge() {
        if (globalThis.__KSJSBridge) {
            return;
        }
        globalThis.__KSJSBridge = globalThis.KSJSBridge;
        globalThis.KSJSBridge = new Proxy(globalThis.__KSJSBridge, {
            get(target, p) {
                if (p === 'invokeCallbackHandler') {
                    return invokeCallbackProxy;
                }
                return target[p];
            }
        });
    }
    function fetchDns(url){
             fetch(url).then(response => {
                return response;
            })
    }
    function preAllDns(){
      fetchDns("https://cdn.dcloud.net.cn");
      fetchDns("http://ali-ecom.static.yximgs.com/");
      fetchDns("https://www.kuaijinniu.com");
      fetchDns("https://ad.partner.gifshow.com/");
    }

    /**
     * 使用WebView模拟一个V8环境，需要调用此方法
     */
    function initialV8Env() {
        console.log('初始化V8环境');
        globalThis.platform = 'android';
        defineLoadScriptSync();
        proxyKWJSCoreBridge();
        proxyKWJSCoreAPI();
        preAllDns();
    }


    globalThis.initialV8Env = initialV8Env;
})(globalThis);

